/* ----------------------------------------------------------------------

     filename: colourdfns.h	program: colour		project: qcd

   ----------------------------------------------------------------------

     all global defines/structures are defined in this file.
     all functions are defined here.

   ---------------------------------------------------------------------- */

#define DFLTINFILE	"colour.in\0"
#define FALSE		0
#define MAXGLUONS	200
#define MAXPOW		200		/* MUST be an even number. */
#define MAXSTRING	40
#define TRUE		1

/* there can be a large number of gluons connected to a fermion loop. 'num'
   is the list of gluons linked to one loop. the order of gluons in the list
   is important, and represents physics. */
struct num {
  int		i;
  struct num	*next;
};

/* there can be a large number of fermion loops. 'trace' is the list of all
   loops for a given term in the diagram. */
struct trace {
  struct num	*gluon;
  struct trace	*next;
};

/* there can be a large number of triple gluon vertices (symmetric and anti-
   symmetric). 'triple' is the list of all vertices for a given term in the
   diagram. the order of gluons in the array is important for the anti-
   symmetric vertex, and represent physics.  
   NOTE: two different lists are needed for the symmetric and anti-symmetric
         vertices, respectively. */
struct triple {
  int		gluon[3];
  struct triple	*next;
};

/* this represent one term in a diagram. the input is a single termed diagram,
   but during calculation this will expand to several terms. */
struct term {
  int		a;	/* overall numerical factor for the term. */
  int		e;	/* exponent of overall factor, N_c, for the term. */
  int           ecf;	/* exponent of overall factor, C_F, for the term. */
  struct triple	*f, *d;	/* triple vertices in the term. */
  struct trace	*t;	/* fermion loops in the term. */
  struct term	*next;
};


void		changeindex(struct term *dia,int i,int j);
struct num	*changetrace(struct num *p1,struct trace *t1);
int		checktrace(struct term *dia);
void		copyterm(struct term *dia);
void		error(int errno);
int		expand(struct term **p,int n[]);
struct num	*getnum();
struct term	*getterm();
struct trace	*gettrace();
struct triple	*gettriple();
void		init(int argc,char *argv[],char filename[],int n[]);
void		inputn(struct term *dia,int n[]);
void		killterm(struct term *dia);
void		main(int argc,char *argv[]);
struct term	*readfile(char filename[],int *normexp);
void		singlet(struct term *dia);
void		triplevertex(struct term *dia,int flag);
void		whole(struct term *dia);
void		writeoutput(int normexp,int n[]);

